/***************************************************************************
 *   Copyright (C) 2008-2009 by Marcel Hasler                              *
 *   mahasler@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kosd.h"
#include "settings.h"

#include <KLocale>
#include <KCmdLineArgs>
#include <KGlobalAccel>
#include <KShortcutsDialog>

#include <fstream>
#include <cstdlib>
#include <iostream>
using namespace std;

KOSD::KOSD()
	: KUniqueApplication("KOSD")
{
	// Keep running when dialog windows are closed
	this->setQuitOnLastWindowClosed(false);

	m_mixer = new Mixer(0);
	m_mixer->setVolumeStep(2);

	m_osd = new OSD();

	m_powermanager = new PowerManager();

	m_actions = new KActionCollection(this);
	m_actions->setConfigGlobal(true);

	// Add volume up
	KAction *newAct = m_actions->addAction("raiseVolume", this);
	newAct->setText("Raise Volume");
	newAct->setGlobalShortcut(KShortcut("Volume Up"), KAction::DefaultShortcut | KAction::ActiveShortcut);
	connect(newAct, SIGNAL(triggered()), this, SLOT(raiseVolume()));

	// Add volume down
	newAct = m_actions->addAction("lowerVolume", this);
	newAct->setText("Lower Volume");
	newAct->setGlobalShortcut(KShortcut("Volume Down"), KAction::DefaultShortcut | KAction::ActiveShortcut);
	connect(newAct, SIGNAL(triggered()), this, SLOT(lowerVolume()));

	// Add volume mute
	newAct = m_actions->addAction("muteVolume", this);
	newAct->setText("Mute Volume");
	newAct->setGlobalShortcut(KShortcut("Volume Mute"), KAction::DefaultShortcut | KAction::ActiveShortcut);
	connect(newAct, SIGNAL(triggered()), this, SLOT(muteVolume()));

	// Add show brightness up
	newAct = m_actions->addAction("showBrightnessUp", this);
	newAct->setText("Show Brightness Up");
	newAct->setGlobalShortcut(KShortcut(), KAction::DefaultShortcut | KAction::ActiveShortcut);
	connect(newAct, SIGNAL(triggered()), this, SLOT(showBrightnessUp()));

	// Add show brightness down
	newAct = m_actions->addAction("showBrightnessDown", this);
	newAct->setText("Show Brightness Down");
	newAct->setGlobalShortcut(KShortcut(), KAction::DefaultShortcut | KAction::ActiveShortcut);
	connect(newAct, SIGNAL(triggered()), this, SLOT(showBrightnessDown()));

	// Add show brightness
	newAct = m_actions->addAction("showBrightness", this);
	newAct->setText("Show Brightness");
	newAct->setGlobalShortcut(KShortcut("Brightness"), KAction::DefaultShortcut | KAction::ActiveShortcut);
	connect(newAct, SIGNAL(triggered()), this, SLOT(showBrightness()));

	// Add show battery charge
	newAct = m_actions->addAction("showBatteryCharge", this);
	newAct->setText("Show Battery Charge");
	newAct->setGlobalShortcut(KShortcut("Battery"), KAction::DefaultShortcut | KAction::ActiveShortcut);
	connect(newAct, SIGNAL(triggered()), this, SLOT(showBattery()));

	m_configDialog = 0;
	m_aboutDialog = 0;

	this->slotApplySettings();
}


KOSD::~KOSD()
{
	delete m_mixer;
	delete m_osd;
	delete m_powermanager;
	delete m_configDialog;
}


int KOSD::newInstance()
{
	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

	if (args->isSet("raiseVolume"))
		this->raiseVolume();
	else if (args->isSet("lowerVolume"))
		this->lowerVolume();
	else if (args->isSet("muteVolume"))
		this->muteVolume();
	else if (args->isSet("showVolume"))
		this->showVolume();
	else if (args->isSet("setVolume"))
		this->setVolume(args->getOption("setVolume").toInt());
	else if (args->isSet("showBrightness"))
		this->showBrightness();
	else if (args->isSet("setBrightness"))
		this->setBrightness(args->getOption("setBrightness").toInt());
	else if (args->isSet("showBattery"))
		this->showBattery();
	else if (args->isSet("configure"))
		this->slotShowConfigDialog();
	else if (args->isSet("about"))
		this->slotShowAboutDialog();
	else if (args->isSet("quit"))
		this->quit();

	args->clear();
	return 0;
}


// Public slots (exposed via DBUS)
void KOSD::showCustom(QString icon, QString label, int percent)
{
	if (percent < 0)
		percent = 0;
	else if (percent > 100)
		percent = 100;

	m_osd->display(icon, label, percent);
}


void KOSD::raiseVolume()
{
	m_mixer->setMute(false);
	unsigned int volume = m_mixer->volumeUp();
	m_osd->display(this->findVolumeIcon(volume), "Volume Up", volume);
}


void KOSD::lowerVolume()
{
	m_mixer->setMute(false);
	unsigned int volume = m_mixer->volumeDown();
	m_osd->display(this->findVolumeIcon(volume), "Volume Down", volume);
}


void KOSD::muteVolume()
{
	bool mute = m_mixer->toggleMute();

	if (mute)
		m_osd->display("audio-volume-muted", "Volume Muted", 0);
	else
	{
		unsigned int volume = m_mixer->volume();
		m_osd->display(this->findVolumeIcon(volume), "Volume Unmuted", volume);
	}
}


void KOSD::showVolume()
{
	bool mute = m_mixer->isMuted();

	if (mute)
		m_osd->display("audio-volume-muted", "Volume Mute", 0);
	else
	{
		unsigned int volume = m_mixer->volume();
		m_osd->display(this->findVolumeIcon(volume), "Volume", volume);
	}
}


void KOSD::setVolume(int volume)
{
	if (volume < 0)
		volume = 0;
	else if (volume > 100)
		volume = 100;

	m_mixer->setMute(false);
	m_osd->display(this->findVolumeIcon(volume), "Volume", m_mixer->setVolume(volume));
}


void KOSD::showBrightnessUp()
{
	m_osd->display("video-display", "Brightness Up", m_powermanager->brightness());
}


void KOSD::showBrightnessDown()
{
	m_osd->display("video-display", "Brightness Down", m_powermanager->brightness());
}


void KOSD::showBrightness()
{
	m_osd->display("video-display", "Brightness", m_powermanager->brightness());
}


void KOSD::setBrightness(int brightness)
{
	if (brightness < 0)
		brightness = 0;
	else if (brightness > 100)
		brightness = 100;

	m_osd->display("video-display", "Brightness", m_powermanager->setBrightness(brightness));
}


void KOSD::showBattery()
{
	if (m_powermanager->batteryState() == PowerManager::Charging)
		m_osd->display(this->findBatteryIcon(), "Battery Charging", m_powermanager->batteryCharge());
	else if (m_powermanager->batteryState() == PowerManager::NoCharge && m_powermanager->batteryCharge() == 100)
		m_osd->display(this->findBatteryIcon(), "Battery Full", m_powermanager->batteryCharge());
	else
		m_osd->display(this->findBatteryIcon(), "Battery Charge", m_powermanager->batteryCharge());
}


// Private slots
void KOSD::slotShowConfigDialog()
{
	if (m_configDialog != 0)
		return;

	m_configDialog = new ConfigurationDialog(Settings::config());
	m_configDialog->setVolumeStep(m_mixer->smallestVolumeStep());
	m_configDialog->show();

	connect(m_configDialog, SIGNAL(shortcutsClicked()), this, SLOT(slotShowShortcutsDialog()));
	connect(m_configDialog, SIGNAL(settingsChanged(const QString&)), this, SLOT(slotApplySettings()));
	connect(m_configDialog, SIGNAL(finished()), this, SLOT(slotConfigDialogFinished()));
}


void KOSD::slotConfigDialogFinished()
{
	m_configDialog->delayedDestruct();
	m_configDialog = 0;
}


void KOSD::slotShowAboutDialog()
{
	if (m_aboutDialog != 0)
		return;

	m_aboutDialog = new KAboutApplicationDialog(KGlobal::mainComponent().aboutData(), 0);
	m_aboutDialog->show();

	connect(m_aboutDialog, SIGNAL(finished()), this, SLOT(slotAboutDialogFinished()));
}


void KOSD::slotAboutDialogFinished()
{
	m_aboutDialog->delayedDestruct();
	m_aboutDialog = 0;
}


void KOSD::slotShowShortcutsDialog()
{
	KShortcutsDialog::configure(m_actions);
}


void KOSD::slotApplySettings()
{
	m_mixer->setVolumeStep(Settings::config()->m_volumeStep);

	m_osd->setSize(Settings::config()->m_size);
	m_osd->setPosition(Settings::config()->m_hPos, Settings::config()->m_vPos);
	m_osd->setOpacity(Settings::config()->m_opacity);
	m_osd->setFakeTransparency(Settings::config()->m_fakeTransparency);
	m_osd->setPrimaryScreenLock((Settings::config()->m_placement == 0) ? true : false);
	m_osd->setTimeout(Settings::config()->m_timeout);
}


QString KOSD::findBatteryIcon()
{
	if (m_powermanager->batteryState() == PowerManager::Discharging)
	{
		if (m_powermanager->batteryCharge() >= 80)
			return QString("battery-100");
		if (m_powermanager->batteryCharge() >= 60)
			return QString("battery-080");
		if (m_powermanager->batteryCharge() >= 40)
			return QString("battery-060");
		if (m_powermanager->batteryCharge() >= 20)
			return QString("battery-040");
		if (m_powermanager->batteryCharge() >= 0)
			return QString("battery-low");
	}

	if (m_powermanager->batteryCharge() >= 80)
		return QString("battery-charging");
	if (m_powermanager->batteryCharge() >= 60)
		return QString("battery-charging-080");
	if (m_powermanager->batteryCharge() >= 40)
		return QString("battery-charging-060");
	if (m_powermanager->batteryCharge() >= 20)
		return QString("battery-charging-040");
	if (m_powermanager->batteryCharge() >= 0)
		return QString("battery-charging-low");
	
	return QString("battery-missing");
}


QString KOSD::findVolumeIcon(unsigned int volume)
{
	if (volume >= 70)
		return QString("audio-volume-high");
	if (volume >= 30)
		return QString("audio-volume-medium");
	return QString("audio-volume-low");
}

#include "kosd.moc"
